#include "mask.h"					//mask header

//note that you can also tweak the viewing paramters in vmaskMain.cpp!!!
float totalScale	= 1.0;			//0.4
float maskWidth		= 63;			//350
float maskHeight	= 82;			//455
float maskDepth		= 63;			//350

extern GLUI_Listbox *three_dPanel;
extern bool displayVM_body;
extern bool displayVM_eyebrows;
extern bool displayVM_mouth;
extern bool displayVM_eyes;

extern int enable3d,disable3d,bushMask,gatesMask,hirstMask,mask1,mask2,mask3;

GLuint LoadTextureRAW( const char * filename, int wrap )
{
	// load a 256x256 RGB .RAW file as a texture

    GLuint texture;
    int width, height;
    char * data;
    FILE * file;

    // open texture data
    file = fopen( filename, "rb" );
    if ( file == NULL ) 
	{
		printf("Texture NOT loaded, because not found");
		return 0;
	}
	else
	{
		//printf("Texture loaded");
	}
    // allocate buffer
    width = 256;
    height = 256;
    data = (char*)malloc( width * height * 3 );

    // read texture data
    fread(data, width * height * 3, 1, file );
    fclose( file );

    // allocate a texture name
    glGenTextures( 1, &texture );

    // select our current texture
    glBindTexture( GL_TEXTURE_2D, texture );

    // select modulate to mix texture with color for shading
    glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);//GL_MODULATE );

    // when texture area is small, bilinear filter the closest mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST );
    // when texture area is large, bilinear filter the first mipmap
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );

    // if wrap is true, the texture wraps over at the edges (repeat)
    //       ... false, the texture ends at the edges (clamp)
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S,  wrap ? GL_REPEAT : GL_CLAMP );
    glTexParameterf( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T,  wrap ? GL_REPEAT : GL_CLAMP );

    // build our texture mipmaps
    gluBuild2DMipmaps( GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data );

    // free buffer
    free( data );

    return texture;
}

void FreeTexture( GLuint texture )
{

  glDeleteTextures( 1, &texture );

}
Point mapTexture(Point p){
	// mapping between world coords and texture
	Point textco;
	textco.x = -(p.x)/63.0+.575; 
	textco.y = p.y/72;
	textco.z = 0;

	return textco;
}
void printSquare(Point p, Point q, Point r, Point s){
	printf("(%f),", p.x,p.y,p.z);
	printf("(%f),",	q.x,q.y,q.z);
	printf("(%f),",	r.x,r.y,r.z);
	printf("(%f)\n",s.x,s.y,s.z);
}

Point getNormalVector(Point p, Point q, Point r){
	Point nv;
	Point s;
	Point t;

	//s = q-p;
	s.x = q.x-p.x;
	s.y = q.y-p.y;
	s.z = q.z-p.z;

	//t = q-r;
	t.x = r.x-q.x;
	t.y = r.y-q.y;
	t.z = r.z-q.z;

	//nv = (s x t); doesnt need normalizing
	nv.x = ((s.y*t.z) - (s.z*t.y)); 
	nv.y = ((s.z*t.x) - (s.x*t.z));
	nv.z = ((s.x*t.y) - (s.y*t.x));

	return nv;
}

Square makeSquare(int startx, int starty, float data[6][13], int scalex, int scaley, int scalez){

	Square test;
	test.lb.x = startx*scalex;
	test.lb.y = starty*scaley;
	test.lb.z = data[startx][starty]*scalez;

	
	test.rb.x = (startx+1)*scalex;
	test.rb.y = starty*scaley;
	test.rb.z = data[startx+1][starty]*scalez;

	
	test.rt.x = (startx+1)*scalex;
	test.rt.y = (starty+1)*scaley;
	test.rt.z = data[startx+1][starty+1]*scalez;

	
	test.lt.x = startx*scalex;
	test.lt.y = (starty+1)*scaley;
	test.lt.z = data[startx][starty+1]*scalez;

	//printSquare(test.lb, test.rb,test.rt,test.lt);
	return test;

}
Square makeSquare2(int startx, int starty, float data[2][2], int scalex, int scaley, int scalez){

	Square test;
	test.lb.x = startx*scalex;
	test.lb.y = starty*scaley;
	test.lb.z = data[0][0]*scalez;

	
	test.rb.x = (startx+1)*scalex;
	test.rb.y = starty*scaley;
	test.rb.z = data[1][0]*scalez;

	
	test.rt.x = (startx+1)*scalex;
	test.rt.y = (starty+1)*scaley;
	test.rt.z = data[1][1]*scalez;

	
	test.lt.x = startx*scalex;
	test.lt.y = (starty+1)*scaley;
	test.lt.z = data[0][1]*scalez;

	//printSquare(test.lb, test.rb,test.rt,test.lt);
	return test;

}

void drawDiamond(Square tempo, GLuint texture){

	Point texco1 = mapTexture(tempo.lb); 
	Point texco2 = mapTexture(tempo.rb);
	Point texco3 = mapTexture(tempo.rt);
	Point texco4 = mapTexture(tempo.lt);

	Point norm		= getNormalVector(tempo.lb,tempo.lt, tempo.rt);
	
	// enable and bind the right texture mapping
    glEnable( GL_TEXTURE_2D );
    glBindTexture( GL_TEXTURE_2D, texture );

	glPushMatrix();
	glBegin(GL_POLYGON);	
		
		glTexCoord2d(texco1.x,texco1.y);
		glColor3f(1,1,1);
		glVertex3f(tempo.lb.x, tempo.lb.y, tempo.lb.z);
		
		glTexCoord2d(texco2.x,texco2.y);
		glColor3f(1,1,1);
		glVertex3f(tempo.rb.x, tempo.rb.y, tempo.rb.z);
		
		glTexCoord2d(texco3.x,texco3.y);
		glColor3f(1,1,1);
		glVertex3f(tempo.rt.x, tempo.rt.y, tempo.rt.z);
		
		glTexCoord2d(texco4.x,texco4.y);
		glColor3f(1,1,1);
		glVertex3f(tempo.lt.x, tempo.lt.y, tempo.lt.z);
		
		glTexCoord2d(texco1.x,texco1.y);
		glColor3f(1,1,1);
		glVertex3f(tempo.lb.x, tempo.lb.y, tempo.lb.z);
	glEnd();	
	
	glPopMatrix();

	glDisable( GL_TEXTURE_2D );
	//glDeleteTextures(1, &texture);
	
}

void drawMask(const char * file){
	//init mask parameters

	//elke masker helft is 5x12 diamonds
	float xScale = 6.3;  
	float yScale = 6.83;  
	float zScale = 7.88;

	int i,j;

	float maskMap[6][13] = {                         //oog          //neus
								{0.0, 2.7, 3.0, 3.3, 2.9, 3.4, 3.8, 4.2, 3.3, 3.6, 3.4, 3.2, 0.0},  // MIDDEN gezicht
								{0.0, 2.6, 2.8, 3.0, 2.7, 3.1, 3.3, 3.7, 3.2, 3.5, 3.3, 3.1, 0.0}, 
								{0.0, 2.3, 2.6, 3.0, 2.6, 3.1, 3.3, 3.6, 3.3, 3.6, 3.1, 2.8, 0.0}, 
								{0.0, 1.8, 2.4, 2.9, 2.5, 3.0, 3.2, 3.5, 3.3, 3.1, 2.9, 0.0, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 1.6, 2.4, 2.8, 2.2, 0.0, 0.0, 0.0, 0.0, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}  // Zijkant (oor/wang)
							};
		

	if (three_dPanel->get_int_val() == disable3d)
	{
		for (i =0; i<6; i++)
		{
			for (j = 0; j<13; j++)
			{
				maskMap[i][j] = 0.0;
			}
		}
	}


	GLuint texture = LoadTextureRAW ( file, 1);

	for(i=0; i<5; i++){
		for(j=0; j<12; j++){

			float newdat[2][2] = {{maskMap[i][j], maskMap[i][j+1]},{maskMap[i+1][j], maskMap[i+1][j+1]}};

			Square tempoLEFT  = makeSquare2(i,j,newdat,xScale,-yScale,zScale);  
			Square tempoRIGHT = makeSquare2(i,j,newdat,-xScale,-yScale,zScale); // gespiegeld over de x as
			drawDiamond(tempoLEFT, texture); // linker helft
			drawDiamond(tempoRIGHT, texture); // rechterhelft
		}
	}
	FreeTexture( texture );
}

void drawMaskHead(const char * file){
	
	//THIS CODE IS NOW NOT IN USE, it was used to model a augmented real face mask with moving parts
	
	//init mask parameters

	//elke masker helft is 5x12 diamonds
	float xScale = 6.3;  
	float yScale = 6.83;  
	float zScale = 7.88;

	int i,j;
float maskMap[6][13] = {                         //oog          //neus
								{0.0, 2.7, 3.0, 3.3, 2.9, 3.4, 3.8, 4.2, 3.3, 3.6, 0.0, 0.0, 0.0},  // MIDDEN gezicht
								{0.0, 2.6, 2.8, 3.0, 2.7, 3.1, 3.3, 3.7, 3.2, 3.5, 0.0, 0.0, 0.0}, 
								{0.0, 2.3, 2.6, 3.0, 2.6, 3.1, 3.3, 3.6, 3.3, 3.6, 3.1, 2.8, 0.0}, 
								{0.0, 1.8, 2.4, 2.9, 2.5, 3.0, 3.2, 3.5, 3.3, 3.1, 2.9, 0.0, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 1.6, 2.4, 2.8, 2.2, 0.0, 0.0, 0.0, 0.0, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}  // Zijkant (oor/wang)
							};
	GLuint texture = LoadTextureRAW ( file, 1);
	for(i=0; i<5; i++){
		for(j=0; j<12; j++){
			float newdat[2][2] = {{maskMap[i][j], maskMap[i][j+1]},{maskMap[i+1][j], maskMap[i+1][j+1]}};
			Square tempoLEFT  = makeSquare2(i,j,newdat,xScale,-yScale,zScale);  
			Square tempoRIGHT = makeSquare2(i,j,newdat,-xScale,-yScale,zScale); // gespiegeld over de x as
			drawDiamond(tempoLEFT, texture); // linker helft
			drawDiamond(tempoRIGHT, texture); // rechterhelft
		}
	}
	FreeTexture( texture );
}

void drawMaskJaw(const char * file){
	
	//THIS CODE IS NOW NOT IN USE, it was used to model a augmented real face mask with moving parts
	
	
	//init mask parameters

	//elke masker helft is 5x12 diamonds
	float xScale = 6.3;  
	float yScale = 6.83;  
	float zScale = 7.88;

	int i,j;

	float maskMap[6][13] = {                         //oog          //neus
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 3.6, 3.4, 3.2, 0.0},  // MIDDEN gezicht
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 3.5, 3.3, 3.1, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 3.6, 3.1, 2.8, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}, 
								{0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}  // Zijkant (oor/wang)
							};
	GLuint texture = LoadTextureRAW ( file, 1);


	for(i=0; i<5; i++){
		for(j=0; j<12; j++){

			float newdat[2][2] = {{maskMap[i][j], maskMap[i][j+1]},{maskMap[i+1][j], maskMap[i+1][j+1]}};


			Square tempoLEFT  = makeSquare2(i,j,newdat,xScale,-yScale,zScale);  
			Square tempoRIGHT = makeSquare2(i,j,newdat,-xScale,-yScale,zScale); // gespiegeld over de x as
			drawDiamond(tempoLEFT, texture); // linker helft
			drawDiamond(tempoRIGHT, texture); // rechterhelft
		}
	}
	FreeTexture( texture );
}

void drawEyeBrows(float EBangle, float EBscale){
	
	//THIS CODE IS NOW NOT IN USE, it was used to model a augmented real face mask with moving parts
	
	float eyebrowAngle = EBangle;
	float scalem = EBscale;

	glDisable( GL_TEXTURE_2D );
	
	glPushMatrix();
		glRotatef(eyebrowAngle,0,0,1);
		glScalef(scalem, scalem, scalem);
		glTranslatef(-2.5,0,0);
		glBegin(GL_QUADS);	
			glColor3f(0.1,0.1,0.1); 
			glVertex3f(0,0,1);
			glVertex3f(1,-1,0);
			glVertex3f(4,-1,0);
			glVertex3f(5,0,1);
			glVertex3f(0,0,1);
			glVertex3f(1,1,0);
			glVertex3f(4,1,0);
			glVertex3f(5,0,1);
		glEnd();
		glTranslatef(2.5,0,0);
		glScalef(1/scalem, 1/scalem, 1/scalem);
		glRotatef(-eyebrowAngle,0,0,1);
	glPopMatrix();
		
		
	
	

}



void vmDrawHelpAxis(void){
	// help axis
	glBegin(GL_LINES);
		glColor3f(1,0,0);
		glVertex3f(0,0,0);
		glVertex3f(20,0,0);
	glEnd();
	glBegin(GL_LINES);
		glColor3f(0,1,0);
		glVertex3f(0,0,0);
		glVertex3f(0,20,0);
	glEnd();
	glBegin(GL_LINES);
		glColor3f(0,0,1);
		glVertex3f(0,0,0);
		glVertex3f(0,0,20);
	glEnd();
}

void vmDrawFace(int type){

	float size				= 10; // face radius
	float quality			= 12; // higher --> more detail
	float eyesize			= size/8;
	float mouthsize			= size/6;
	float eyebrowwidth		= size/10;
	float eyebrowlength		= size/2;
	float eyebrowangle		= 10;
	float eyebrowlift		= 10;
	float mouthcorner		= 0; // 1,0,-1

	
	GLUquadric* obj;
	

		switch(type)
			{
			case 0: // sad
				{
					eyebrowangle	=	 20;
					eyebrowlift		=	-10;
					mouthcorner		=	-1;
				}
				break;
			case 1:  // angry
				{	
					eyebrowangle	=	-10;
					eyebrowlift		=	-10;
					mouthcorner		=	-1;
				}
				break;
			case 2:  // happy
				{
					eyebrowangle	=	0;
					eyebrowlift		=	10;
					mouthcorner		=	1;
				}
				break;
			case 3: // neutral
				{	
					eyebrowangle	=	0;
					eyebrowlift		=	0;
					mouthcorner		=	0;
				}
				break;
			case 4:	
				{}
				break;
			default:
				{}
			}

	//vmDrawHelpAxis();

	
	glRotatef(-90.0, 1,0,0); // mask must be parallel to the marker
	
		
		
		// draw mask body
		if (displayVM_body ==true)
		{
			glColor3f(1.0,0.8,0.8);
			glutSolidSphere(size,quality*size,quality*size);
			obj = gluNewQuadric();
			gluCylinder(obj, size, size+0.1, size, quality*size, quality*size);
			glTranslatef(0,0,size);
			glColor3f(0.2,0.2,0.9); //hair
			glutSolidSphere(size,quality*size,quality*size); 
		}

		// draw eyes
		if (displayVM_eyes ==true)
		{
			glColor3f(0.1,0.1,0.1); //eye
			glTranslatef(0,-size,-size/4);

			glTranslatef(size/4,0,0); glRotatef(90,1,0,0); glutSolidTorus(eyesize/2, eyesize, quality*eyesize,quality*eyesize); glRotatef(-90,1,0,0);
			glTranslatef(-size/2,0,0); glRotatef(90,1,0,0); glutSolidTorus(eyesize/2, eyesize, quality*eyesize,quality*eyesize); glRotatef(-90,1,0,0);
			
			glTranslatef(size/4,0,0);	//terug naar centrum hoofddeksel
			glTranslatef(0,size,size/4);
		}

		// draw mouth
		if (displayVM_mouth == true)
		{
			glColor3f(1,1,1);
			glTranslatef(0,-1.1*size, -size);
			glBegin(GL_QUADS);
				glVertex3f(-4,0,mouthcorner+1); 
				glVertex3f(-1,0,0);
				glVertex3f(-1,0,1);
				glVertex3f(-4,0,mouthcorner+1); 

				glVertex3f(-1,0,0);
				glVertex3f(1,0,0);
				glVertex3f(1,0,1);
				glVertex3f(-1,0,1);

				glVertex3f(4,0,mouthcorner+1); 
				glVertex3f(1,0,0);
				glVertex3f(1,0,1);
				glVertex3f(4,0,mouthcorner+1); 
				
				//lips red
				glColor3f(0.7,0.3,0.3); 

				glVertex3f(-4,0,mouthcorner+1); 
				glVertex3f(-1,0,1);
				glVertex3f(-1,0,2);
				glVertex3f(-4,0,mouthcorner+1);

				glVertex3f(-4,0,mouthcorner+1); 
				glVertex3f(-1,0,-1);
				glVertex3f(-1,0,0);
				glVertex3f(-4,0,mouthcorner+1);

				glVertex3f(-1,0,1);
				glVertex3f(1,0,1);
				glVertex3f(1,0,2);
				glVertex3f(-1,0,2);

				glVertex3f(-1,0,0);
				glVertex3f(1,0,0);
				glVertex3f(1,0,-1);
				glVertex3f(-1,0,-1);

				glVertex3f(4,0,mouthcorner+1); 
				glVertex3f(1,0,1);
				glVertex3f(1,0,2);
				glVertex3f(4,0,mouthcorner+1); 

				glVertex3f(4,0,mouthcorner+1); 
				glVertex3f(1,0, 0);
				glVertex3f(1,0,-1);
				glVertex3f(4,0,mouthcorner+1); 
			glEnd();
			glTranslatef(0, size/1.1, size);
		}


		// draw eyebrows
		if (displayVM_eyebrows==true)
		{
			glColor3f(0.2,0.1,0); //eyebrow
			glTranslatef(0,-(size),size/8); 
			glTranslatef(size/4,0,0); //wb1 
			glTranslatef(0,0,eyebrowlift/100*size);

			glRotatef(eyebrowangle,0,1,0);

				glBegin(GL_QUADS);
					glVertex3f(-eyebrowlength/2,0,0.6*eyebrowwidth/2);
					glVertex3f(-eyebrowlength/2,0,-0.6*eyebrowwidth/2);

					glVertex3f(0,0,0);
					glVertex3f(0,0,eyebrowwidth);

					glVertex3f(0,0,eyebrowwidth);
					glVertex3f(0,0,0);

					glVertex3f(eyebrowlength/2,0,-0.6*eyebrowwidth/2);
					glVertex3f(eyebrowlength/2,0,0.6*eyebrowwidth/2);
				glEnd();
				
			glRotatef(-eyebrowangle,0,1,0);
			glTranslatef(-size/2,0,0); //wb2
			glRotatef(-eyebrowangle,0,1,0);
				glBegin(GL_QUADS);
					glVertex3f(-eyebrowlength/2,0,0.6*eyebrowwidth/2);
					glVertex3f(-eyebrowlength/2,0,-0.6*eyebrowwidth/2);

					glVertex3f(0,0,0);
					glVertex3f(0,0,eyebrowwidth);

					glVertex3f(0,0,eyebrowwidth);
					glVertex3f(0,0,0);

					glVertex3f(eyebrowlength/2,0,-0.6*eyebrowwidth/2);
					glVertex3f(eyebrowlength/2,0,0.6*eyebrowwidth/2);
				glEnd();
			glRotatef(eyebrowangle,0,1,0);
			glTranslatef(0,0,-eyebrowlift/100*size);
			glTranslatef(size/4,0,0);	 //terug naar centrum hoofddeksel
			glTranslatef(0,size,-size/8);
		}


	glRotatef(90,1,0,0);


}
