/* This file uses simpleLite.c of Philip Lamb as a basis. It has been merged with loadMultiple.c from the 
ARToolkit examples ((by Dr. Hirokazu Kato)) to enable multiple markers. The application uses the GLUI library for UI control. Tnx guys.

All example code has been severly extended by Steven Bos and Hannes Smit for a  course 3D Computer graphics+VR, TU Delft.
 */

// ============================================================================
//	Includes
// ============================================================================

#include <stdio.h>
#include <stdlib.h>				// malloc(), free()
#ifdef __APPLE__
#  include <GLUT/glut.h>
#else
#  include <GL/glut.h>
#endif
#include <AR/config.h>
#include <AR/video.h>
#include <AR/param.h>			// arParamDisp()
#include <AR/ar.h>
#include <AR/gsub_lite.h>		//enables ARtoolkit in combination with opengl renderer
#include "object.h"				//read multiple markers
#include "mask.h"				//mask header
#include <GL/glui.h>			//glui support

GLUI *glui;
GLUI_RadioGroup *modus;
GLUI_Listbox *modelPanel, *three_dPanel;

int enable3d =0;
int disable3d =1;

int bushMask =0;
int gatesMask =1;
int hirstMask =2;

int mask1=0;
int mask2=1;
int mask3=2;
int mask4=3;

bool displayVM_body			= false;
bool displayVM_eyebrows		= false;
bool displayVM_mouth		= false;
bool displayVM_eyes			= false;

GLUI_RadioButton *aniMaskButton, *singleMaskButton, *permMaskButton;
GLUI_RadioGroup *modusRadioGroup;


/********** UI control ID definitions for callbacks  ********/
#define MODUS_ID		101
#define MODEL_ID		102
#define THREED_ID		103

// ============================================================================
//	Constants 
// ============================================================================
//NOTE THESE ARE VIEWING PARAMETERS, TWEAK THEM WHEN AN OBJECT IS NOT VISIBLE
#define VIEW_SCALEFACTOR		0.188			// 1.0 ARToolKit unit becomes 0.025 of my OpenGL units. -> I set it back to 1.0 so it is the same now.
#define VIEW_DISTANCE_MIN		0.1			// Objects closer to the camera than this will not be displayed.
#define VIEW_DISTANCE_MAX		1000.0		// Objects further away from the camera than this will not be displayed.

//VIEW_SCALEFACTOR = height of pattern / height of mask (in reality) : 47/250 = 0.188

// ============================================================================
//	Global variables
// ============================================================================

//window id
int main_window;

//UI control value
int		modusBox		= 0;
int		modelBox		= 0;
int		threedBox		= 0;
float	xy_aspect;

//external mask variables (can be changes in mask.cpp)
extern float totalScale;
extern float maskWidth;
extern float maskHeight;
extern float maskDepth;

// Preferences.
static int prefWindowed = TRUE;
static int prefWidth = 640;					// Fullscreen mode width.
static int prefHeight = 480;				// Fullscreen mode height.
static int prefDepth = 32;					// Fullscreen mode bit depth.
static int prefRefresh = 0;					// Fullscreen mode refresh rate. Set to 0 to use default rate.

// Image acquisition.
static ARUint8		*gARTImage = NULL;

// Marker detection.
static int			gARTThreshhold = 100;
static long			gCallCountMarkerDetect = 0;

// Transformation matrix retrieval.
static double		gPatt_trans[3][4];		
static int			gPatt_found = FALSE;	

// Drawing.
static ARParam		gARTCparam;
static ARGL_CONTEXT_SETTINGS_REF gArglSettings = NULL;
static int gDrawRotate = FALSE;
float gDrawRotateAngle = 0;					// For use in drawing.

// Object Data
char            *model_name = "Data/object_data2";
ObjectData_T    *object;
int             objectnum;

//Forward Function declarations
void draw_object(int obj_id);
void GUImode(void);



// ============================================================================
//	Functions
// ============================================================================

void control_cb( int control )
{
	switch(control){
		case MODUS_ID:
			{
			GUImode();
			}
		break;
		case MODEL_ID:
			{}
		break;
		case THREED_ID:
			{}
		break;
		default: { 
			//niets
		}
	}
}

static void	initLights(void){
	// onderstaande lichtmodel is overgenomen vanuit mountain project
	/* lichtpunt 1 */
	GLfloat lightAmbient1[4] =			{1, 1, 1, 1};			//kleur van ambient licht
	GLfloat lightDiffuse1[4] =			{1.0f, 1.0f, 1.0f, 0.5f};		//kleur van het licht
	GLfloat lightSpecular1[4] =			{1.0, 1.0, 1.0, 1.0};			//highlights
	GLfloat lightPosition1[4] =			{0.0, 0.0, 10.0, 1.0};
	GLfloat lightSpotDirection1[3] =	{0.0, 0.0, -10.0 };				//waar de spot naartoe schijnt
	/* lichtpunt 2 */
	GLfloat lightAmbient2[4] =			{1, 1, 1, 1};			//kleur van ambient licht
	GLfloat lightDiffuse2[4] =			{1.0f, 1.0f, 1.0f, 0.5f};		//kleur van het licht
	GLfloat lightSpecular2[4] =			{1.0, 1.0, 1.0, 1.0};			//highlights
	GLfloat lightPosition2[4] =			{0.0, 0.0, -10.0, 1.0};
	GLfloat lightSpotDirection2[3] =	{0.0, 0.0, 10.0 };				//waar de spot naartoe schijnt
	/* enable all */ 
	glEnable(GL_LIGHT0);
	glEnable(GL_COLOR_MATERIAL);
	glLightfv(GL_LIGHT0,	GL_AMBIENT,					lightAmbient1);	
	glLightfv(GL_LIGHT0,	GL_DIFFUSE,					lightDiffuse1);
	glLightfv(GL_LIGHT0,	GL_SPECULAR,				lightSpecular1);
	glLightfv(GL_LIGHT0,	GL_POSITION,				lightPosition1);
	glLightfv(GL_LIGHT0,	GL_SPOT_DIRECTION,			lightSpotDirection1);
	glEnable(GL_LIGHT1);
	glLightfv(GL_LIGHT1,	GL_AMBIENT,					lightAmbient2);
	glLightfv(GL_LIGHT1,	GL_DIFFUSE,					lightDiffuse2);
	glLightfv(GL_LIGHT1,	GL_SPECULAR,				lightPosition2);
	glLightfv(GL_LIGHT1,	GL_POSITION,				lightSpecular2);
	glLightfv(GL_LIGHT1,	GL_SPOT_DIRECTION,			lightSpotDirection2);
	glEnable(GL_LIGHTING);
	glShadeModel(GL_SMOOTH);	//glShadeModel(GL_FLAT);
}

static int setupCamera(const char *cparam_name, char *vconf, ARParam *cparam)
{	
    ARParam			wparam;
	int				xsize, ysize;

    // Open the video path.
    if (arVideoOpen(vconf) < 0) {
    	fprintf(stderr, "setupCamera(): Unable to open connection to camera.\n");
    	return (FALSE);
	}
	
    // Find the size of the window.
    if (arVideoInqSize(&xsize, &ysize) < 0) return (FALSE);
    fprintf(stdout, "Camera image size (x,y) = (%d,%d)\n", xsize, ysize);
	
	// Load the camera parameters, resize for the window and init.
    if (arParamLoad(cparam_name, 1, &wparam) < 0) {
		fprintf(stderr, "setupCamera(): Error loading parameter file %s for camera.\n", cparam_name);
        return (FALSE);
    }
    arParamChangeSize(&wparam, xsize, ysize, cparam);
    fprintf(stdout, "*** Camera Parameter ***\n");
    arParamDisp(cparam);
	
    arInitCparam(cparam);

	if (arVideoCapStart() != 0) {
    	fprintf(stderr, "setupCamera(): Unable to begin camera data capture.\n");
		return (FALSE);		
	}
	
	return (TRUE);
}

// Report state of ARToolKit global variables arFittingMode,
// arImageProcMode, arglDrawMode, arTemplateMatchingMode, arMatchingPCAMode.
static void debugReportMode(void)
{
	if(arFittingMode == AR_FITTING_TO_INPUT ) {
		fprintf(stderr, "FittingMode (Z): INPUT IMAGE\n");
	} else {
		fprintf(stderr, "FittingMode (Z): COMPENSATED IMAGE\n");
	}
	
	if( arImageProcMode == AR_IMAGE_PROC_IN_FULL ) {
		fprintf(stderr, "ProcMode (X)   : FULL IMAGE\n");
	} else {
		fprintf(stderr, "ProcMode (X)   : HALF IMAGE\n");
	}
	
	if (arglDrawModeGet(gArglSettings) == AR_DRAW_BY_GL_DRAW_PIXELS) {
		fprintf(stderr, "DrawMode (C)   : GL_DRAW_PIXELS\n");
	} else if (arglTexmapModeGet(gArglSettings) == AR_DRAW_TEXTURE_FULL_IMAGE) {
		fprintf(stderr, "DrawMode (C)   : TEXTURE MAPPING (FULL RESOLUTION)\n");
	} else {
		fprintf(stderr, "DrawMode (C)   : TEXTURE MAPPING (HALF RESOLUTION)\n");
	}
		
	if( arTemplateMatchingMode == AR_TEMPLATE_MATCHING_COLOR ) {
		fprintf(stderr, "TemplateMatchingMode (M)   : Color Template\n");
	} else {
		fprintf(stderr, "TemplateMatchingMode (M)   : BW Template\n");
	}
	
	if( arMatchingPCAMode == AR_MATCHING_WITHOUT_PCA ) {
		fprintf(stderr, "MatchingPCAMode (P)   : Without PCA\n");
	} else {
		fprintf(stderr, "MatchingPCAMode (P)   : With PCA\n");
	}
}

static void Quit(void)
{
	arglCleanup(gArglSettings);
	arVideoCapStop();
	arVideoClose();
	exit(0);
}

static void Keyboard(unsigned char key, int x, int y)
{
	int mode;
	switch (key) {
		case 0x1B:						// Quit.
		case 'Q':
		case 'q':
			Quit();
			break;
		case ' ':
			gDrawRotate = !gDrawRotate;
			break;
		case 'C':
		case 'c':
			mode = arglDrawModeGet(gArglSettings);
			if (mode == AR_DRAW_BY_GL_DRAW_PIXELS) {
				arglDrawModeSet(gArglSettings, AR_DRAW_BY_TEXTURE_MAPPING);
				arglTexmapModeSet(gArglSettings, AR_DRAW_TEXTURE_FULL_IMAGE);
			} else {
				mode = arglTexmapModeGet(gArglSettings);
				if (mode == AR_DRAW_TEXTURE_FULL_IMAGE)	arglTexmapModeSet(gArglSettings, AR_DRAW_TEXTURE_HALF_IMAGE);
				else arglDrawModeSet(gArglSettings, AR_DRAW_BY_GL_DRAW_PIXELS);
			}
			fprintf(stderr, "*** Camera - %f (frame/sec)\n", (double)gCallCountMarkerDetect/arUtilTimer());
			gCallCountMarkerDetect = 0;
			arUtilTimerReset();
			debugReportMode();
			break;
		case 'D':
		case 'd':
			arDebug = !arDebug;
			break;
		case '?':
		case '/':
			printf("Keys:\n");
			printf(" q or [esc]    Quit demo.\n");
			printf(" c             Change arglDrawMode and arglTexmapMode.\n");
			printf(" d             Activate / deactivate debug mode.\n");
			printf(" ? or /        Show this help.\n");
			printf("\nAdditionally, the ARVideo library supplied the following help text:\n");
			arVideoDispOption();
			break;
		default:
			break;
	}
}

static void Idle(void)
{
	static int ms_prev;
	int ms;
	float s_elapsed;
	
	ARUint8 *image;
	ARMarkerInfo    *marker_info;					// Pointer to array holding the details of detected markers.
    int             marker_num;						// Count of number of markers detected.
    int             i, j, k;
	
	// Find out how long since Idle() last ran.
	ms = glutGet(GLUT_ELAPSED_TIME);
	s_elapsed = (float)(ms - ms_prev) * 0.001;
	if (s_elapsed < 0.01f) return; // Don't update more often than 100 Hz.
	ms_prev = ms;
	
	// Grab a video frame.
	if ((image = arVideoGetImage()) != NULL) {
		gARTImage = image;	// Save the fetched image.
		gPatt_found = FALSE;	// Invalidate any previous detected markers.
		
		gCallCountMarkerDetect++; // Increment ARToolKit FPS counter.
		
		// Detect the markers in the video frame.
		if (arDetectMarker(gARTImage, gARTThreshhold, &marker_info, &marker_num) < 0) {
			exit(-1);
		}
		
		// Check through the marker_info array for highest confidence
		// visible marker matching our preferred pattern.
	
	/* check for known patterns */
    for( i = 0; i < objectnum; i++ ) {
		k = -1;
		for( j = 0; j < marker_num; j++ ) {
	        if( object[i].id == marker_info[j].id) {

				/* you've found a pattern */
				//printf("Found pattern: %d ",patt_id)

				if( k == -1 ) k = j;
		        else /* make sure you have the best pattern (highest confidence factor) */
					if( marker_info[k].cf < marker_info[j].cf ) k = j;
			}
		}
		if( k == -1 ) {
			object[i].visible = 0;
			continue;
		}
		
		/* calculate the transform for each marker */
		if( object[i].visible == 0 ) {
            arGetTransMat(&marker_info[k],
                          object[i].marker_center, object[i].marker_width,
                          object[i].trans);
        }
        else {
            arGetTransMatCont(&marker_info[k], object[i].trans,
                          object[i].marker_center, object[i].marker_width,
                          object[i].trans);
			gPatt_found = TRUE;
        }
        object[i].visible = 1;
	}
		

		// Tell GLUT the display has changed.
		glutSetWindow(main_window);
		glutPostRedisplay();
	}
}

//
//	This function is called on events when the visibility of the
//	GLUT window changes (including when it first becomes visible).
//
static void Visibility(int visible)
{
	if (visible == GLUT_VISIBLE) {
		GLUI_Master.set_glutIdleFunc (Idle);
	} else {
		GLUI_Master.set_glutIdleFunc (NULL);
	}
}

//
//	This function is called when the GLUT window is resized.
//
static void Reshape(int w, int h)
{	
	int tx, ty, tw, th;
	GLUI_Master.get_viewport_area( &tx, &ty, &tw, &th );
	glViewport( tx, ty, tw, th );

	xy_aspect = (float)tw / (float)th;

	glutPostRedisplay();
}

//
// This function is called when the window needs redrawing.
//
static void Display(void)
{
    GLdouble p[16];
	GLdouble m[16];
	int i;
		
	glDrawBuffer(GL_BACK);										// Select correct buffer for this context.
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);			// Clear the buffers for new frame.

	arglDispImage(gARTImage, &gARTCparam, 1.0, gArglSettings);	// zoom = 1.0.
	arVideoCapNext();
	gARTImage = NULL;											// Image data is no longer valid after calling arVideoCapNext().
	
	if (gPatt_found) {
		// Projection transformation.
		arglCameraFrustumRH(&gARTCparam, VIEW_DISTANCE_MIN, VIEW_DISTANCE_MAX, p);
		glMatrixMode(GL_PROJECTION);
		glLoadMatrixd(p);
		glMatrixMode(GL_MODELVIEW);
		
		// Viewing transformation.
		glLoadIdentity();

		for( i = 0; i < objectnum; i++ ) {
			if( object[i].visible == 0 ) continue;
				arglCameraViewRH(object[i].trans, m, VIEW_SCALEFACTOR);
				glLoadMatrixd(m);
				draw_object( object[i].id);
		}
     
	} // gPatt_found

int type;
type = modelPanel->get_int_val();

switch (modusRadioGroup->get_int_val())
	{
	case 0: //single mask modus
		{
				//do nothing and let "gpatt found" handle augmented reality
		}
		break;
	case 1: //animated mask modus
		{
		glPushMatrix(); // Save world coordinate system.			
			vmDrawFace(type);
		glPopMatrix();	// Restore world coordinate system.

		//disable permanent overlay withouth marker detection
		displayVM_body		= false;
		displayVM_eyebrows	= false;
		displayVM_mouth		= false;
		displayVM_eyes		= false;
		}
		break;
	case 2: //permanent mask modus
		{
		glPushMatrix(); // Save world coordinate system.			
			vmDrawFace(type);
		glPopMatrix();	// Restore world coordinate system.
		}
		break;
	default:
		{}
		//nothing
	}

	glutSwapBuffers();
}

void draw_object( int obj_id)
{
	switch (modusRadioGroup->get_int_val())
	{
	case 0: //single mask modus
		{
			switch(obj_id)
				{
				case 0: //bush patroon
					{
						glPushMatrix(); // Save world coordinate system.
						glTranslatef( 0.0, maskHeight/2*totalScale, 20.0 ); // Place base of cube on marker surface
						drawMask("bush.bmp");
						glPopMatrix();	// Restore world coordinate system.
					}
					break;
				case 1: //gates patroon
					{
						glPushMatrix(); // Save world coordinate system.
						glTranslatef( 0.0, maskHeight/2*totalScale, 20.0 ); // Place base of cube on marker surface
						drawMask("gates.bmp");
						glPopMatrix();	// Restore world coordinate system.
					}
					break;
				case 2: //hirst patroon
					{
						glPushMatrix(); // Save world coordinate system.
						glTranslatef( 0.0, maskHeight/2*totalScale, 20.0 ); // Place base of cube on marker surface
						drawMask("hirst.bmp");
						glPopMatrix();	// Restore world coordinate system.
					}
					break;
				default:
					{}
				}		
		}
		break;
	case 1: //animated mask modus
		{
		switch(obj_id)
			{
			case 3: //vm body patroon
				{
					displayVM_body	= true;
				}
				break;
			case 4: //vm eyes patroon
				{
					displayVM_eyes	= true;
				}
				break;
			case 5: //vm mouth patroon
				{
					displayVM_mouth	= true;
				}
				break;
			case 6: //vm eyebrows patroon
				{
					displayVM_eyebrows	= true;
				}
				break;
			default:
				{}
			}		
		}
		break;
	case 2: //permanent mask modus
		{
		switch(obj_id)
			{
			case 3: //vm body patroon
				{
					displayVM_body	= true;
				}
				break;
			case 4: //vm eyes patroon
				{
					displayVM_eyes	= true;
				}
				break;
			case 5: //vm mouth patroon
				{
					displayVM_mouth	= true;
				}
				break;
			case 6: //vm eyebrows patroon
				{
					displayVM_eyebrows	= true;
				}
				break;
			case 7: // delete all patroon
				{
					displayVM_body		= false;
					displayVM_eyebrows	= false;
					displayVM_mouth		= false;
					displayVM_eyes		= false;
				}
				break;
			default:
				{}
			}	
		}
			break;
	default:
		{}
		//nothing
	}
}

void GUImode(void)
{
	int id =0;
	displayVM_body		= false;
	displayVM_eyebrows	= false;
	displayVM_mouth		= false;
	displayVM_eyes		= false;

	switch (modusRadioGroup->get_int_val())
	{
	case 0: //single mask modus
		{
			modelPanel->disable();
			three_dPanel->enable();
		}
		break;
	case 1: //animated mask modus
		{
			while (modelPanel->delete_item(id))
			{
				id++;	
			}

			modelPanel->add_item(mask1,"Sad");
			modelPanel->add_item(mask2,"Angry");
			modelPanel->add_item(mask3,"Happy");
			modelPanel->add_item(mask4,"Neutral");

			three_dPanel->disable();
			modelPanel->enable();
		}
		break;
	case 2: //permanent mask modus
		{
			while (modelPanel->delete_item(id))
			{
				id++;	
			}

			modelPanel->add_item(mask1,"Sad");
			modelPanel->add_item(mask2,"Angry");
			modelPanel->add_item(mask3,"Happy");
			modelPanel->add_item(mask4,"Neutral");

			three_dPanel->disable();
			modelPanel->enable();
		}
		break;
	default:
		{}
		//nothing
	}
}


int main(int argc, char** argv)
{
	char glutGamemode[32];
	const char *cparam_name = "Data/camera_para.dat";
	//
	// Camera configuration.
	//
#ifdef _WIN32
	char			*vconf = "Data\\WDM_camera_flipV.xml";
#else
	char			*vconf = "";
#endif
	
	// ----------------------------------------------------------------------------
	// Library and Light inits 
	//

	glutInit(&argc, argv);
	initLights();

	// ----------------------------------------------------------------------------
	// Hardware setup.
	//

	if (!setupCamera(cparam_name, vconf, &gARTCparam)) {
		fprintf(stderr, "main(): Unable to set up AR camera.\n");
		exit(-1);
	}

	//  ---------------------------------------------------------------------------
	// Load in the object data - trained markers and associated bitmap files
	//
    if( (object=read_ObjData(model_name, &objectnum)) == NULL ) exit(0);
		printf("Objectfile num = %d\n", objectnum);
	
	// ----------------------------------------------------------------------------
	// Library setup.
	//

	// Set up GL context(s) for OpenGL to draw into.
	glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
	if (!prefWindowed) {
		if (prefRefresh) sprintf(glutGamemode, "%ix%i:%i@%i", prefWidth, prefHeight, prefDepth, prefRefresh);
		else sprintf(glutGamemode, "%ix%i:%i", prefWidth, prefHeight, prefDepth);
		glutGameModeString(glutGamemode);
		glutEnterGameMode();
	} else {
		glutInitWindowSize(prefWidth, prefHeight);
		main_window = glutCreateWindow( "Virtual mask by Steven & Hannes" );

	}

	// Setup argl library for current context.
	if ((gArglSettings = arglSetupForCurrentContext()) == NULL) {
		fprintf(stderr, "main(): arglSetupForCurrentContext() returned error.\n");
		exit(-1);
	}
	debugReportMode();
	glEnable(GL_DEPTH_TEST);
	arUtilTimerReset();
	
	/****************************************/
	/*       Set up OpenGL texture          */
	/****************************************/

	//glEnable( GL_TEXTURE_2D );
	//LoadTextureRAW ( "hirst.bmp", 1);
	//LoadTextureRAW ( "bush.bmp", 1);
	glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT); 

	// Register GLUT event-handling callbacks.
	// NB: Idle() is registered by Visibility.
	glutDisplayFunc(Display);
	glutVisibilityFunc(Visibility);
	glutKeyboardFunc(Keyboard);
	
	GLUI_Master.set_glutReshapeFunc(Reshape);  

	/***************************************************************/
	/*         Here's the GLUI code  (om oa. UI controls te bouwen)*/
	/***************************************************************/
	
	/*** Create the bottom subwindow ***/
	glui = GLUI_Master.create_glui_subwindow( main_window, GLUI_SUBWINDOW_BOTTOM );

	GLUI_Panel *modusPanel = glui->add_panel("Mode:",GLUI_PANEL_EMBOSSED);

	modusRadioGroup = new GLUI_RadioGroup(modusPanel,&modusBox ,MODUS_ID, control_cb);

	singleMaskButton = new GLUI_RadioButton(modusRadioGroup, "Realface Mask" );
	aniMaskButton = new GLUI_RadioButton(modusRadioGroup, "Animated Mask" );
	permMaskButton = new GLUI_RadioButton(modusRadioGroup, "Permanent Mask" );
	
	glui->add_column(false);

	GLUI_Panel *optionPanel = glui->add_panel("Virtual Mask options:",GLUI_PANEL_EMBOSSED);
	
	modelPanel = new GLUI_Listbox(optionPanel, "Choose Model: ",&modelBox ,MODEL_ID, control_cb );
	
	glui->add_column_to_panel(optionPanel,false);

	three_dPanel = new GLUI_Listbox(optionPanel, "Enable 3D mask",&threedBox ,THREED_ID, control_cb );
			three_dPanel->add_item(enable3d,"Enable");
			three_dPanel->add_item(disable3d,"Disable");

	GLUI_Master.sync_live_all();
	GUImode(); 

		/**** Regular GLUT main loop ****/
	glui->set_main_gfx_window(main_window);

	glutMainLoop();

	return (0);
}
